@@ -785,6 +785,21 @@ class Panoply:
         else:
             return False
 
+    def has_running_jobs(self) -> bool:
+        """
+        Simple check to determine if there are any running jobs on this device
+
+        :return: bool True if there is currently a running job
+        """
+
+        self.execute_cli('show jobs all')
+        jobs = self.xapi.xml_document
+        jobs_element = etree.fromstring(jobs)
+
+        running_jobs_list = jobs_element.xpath(".//jobs/status[text() != 'FIN']")
+
+        return True if running_jobs_list else False
+
     def wait_for_device_ready(self, interval=30, timeout=600) -> bool:
         """
         Loop and wait until device is ready or times out
@@ -803,10 +818,25 @@ class Panoply:
         while True:
             try:
 
-                self.xapi.op(cmd='<show><chassis-ready></chassis-ready></show>')
+                # fix for #60 - show chassis ready is not available on panorama
+                if self.facts.get('model', 'Panorama') == 'Panorama':
+                    cmd = "<show><system><info></info></system></show>"
+                    is_panorama = True
+                else:
+                    cmd = '<show><chassis-ready></chassis-ready></show>'
+                    is_panorama = False
+
+                self.xapi.op(cmd=cmd)
                 resp = self.xapi.xml_result()
 
                 if self.xapi.status == 'success':
+                    # in the case of panorama, we may be up but the auto-commit job may still be running
+                    # continue to wait until there are no more jobs
+                    # FIXME - should probably enhance this to only check for auto-commit job, it's possible there
+                    # can be other jobs running with a busy / heavily used panorama instance
+                    if is_panorama:
+                        if not self.has_running_jobs():
+                            return True
 
                     if resp.strip() == 'yes':
                         return True
@@ -1050,6 +1080,64 @@ class Panoply:
             logger.error('Could not get configuration from device')
             raise PanoplyException('Could not get configuration from the device')
 
+    def get_saved_configuration(self, configuration_name: str) -> str:
+        """
+        Returns a saved configuration on the device. Use 'list_saved_configuration' to get a list of available options
+
+        :param configuration_name: name of the saved configuration to export
+        :return: configuration as an XML encoded string
+        """
+
+        try:
+
+            if self.connected:
+                self.xapi.op(f'<show><config><saved>{configuration_name}</saved></config></show>', cmd_xml=False)
+
+                if self.xapi.status != 'success':
+                    raise PanoplyException('Could not get saved configuration from the device')
+
+                doc_str = self.xapi.xml_result()
+
+                doc = etree.fromstring(doc_str)
+                return etree.tostring(doc, pretty_print=True).decode(encoding='UTF-8')
+
+            else:
+                return ''
+
+        except PanXapiError:
+            logger.error('Could not get saved configuration from device')
+            raise PanoplyException('Could not get saved configuration from the device')
+
+    def list_saved_configurations(self) -> list:
+        """
+        Returns a list of saved configuration files on this device
+
+        :return: list of saved configurations
+        """
+
+        saved_configurations = list()
+        try:
+            if not self.connected:
+                return saved_configurations
+
+            self.xapi.ad_hoc(qs='type=op&action=complete&xpath=/operations/load/config/from', modify_qs=True)
+
+            if self.xapi.status != 'success':
+                raise PanoplyException('Could not list saved configurations from the device')
+
+            response_root = self.xapi.element_root
+            config_items = response_root.findall('.//completion')
+            for config in config_items:
+                if 'value' in config.attrib:
+                    saved_configurations.append(config.attrib['value'])
+
+            return saved_configurations
+
+        except PanXapiError as pe:
+            logger.error('Could not list configurations from device')
+            logger.error(pe)
+            raise PanoplyException('Could not list saved configuration from the device')
+
     def generate_skillet(self, from_candidate=False) -> list:
         """
         Generates a skillet from the changes detected on this device.
@@ -1105,16 +1193,23 @@ class Panoply:
 
         for xpath in not_found_xpaths:
 
-            set_xpath, entry = self.__split_xpath(xpath)
-            # force full xpath for #52
-            set_xpath = re.sub(r'^\./', r'/config/', set_xpath)
-            full_relative_xpath = re.sub(r'^/config/', r'\./', xpath)
+            # xpath comes as a relative full xpath like './mgt-config/password-complexity'
+            # make it /config/mgt-config/password-complexity
+            full_xpath = re.sub(r'^\./', r'/config/', xpath)
+            # force full xpath for #52 - split off last bit like password-complexity
+            set_xpath, entry = self.__split_xpath(full_xpath)
+            # remove any attribute for tag name
             tag = re.sub(r'\[.*\]', '', entry)
 
             # check if this xpath is actually user configurable
             if self.__is_ignored_xpath(set_xpath):
                 continue
 
+            # in some cases we need to ignore xpath + element for example
+            # /config/shared/content-preview
+            if self.__is_ignored_xpath(full_xpath):
+                continue
+
             changed_elements = latest_doc.xpath(xpath)
 
             if not changed_elements:
@@ -1122,8 +1217,9 @@ class Panoply:
                 continue
 
             changed_element = changed_elements[0]
-            # FIXME - do we need to __clean_uuid() here ?
-            xml_string = etree.tostring(changed_element).decode(encoding='UTF-8')
+
+            cleaned_element = self.__clean_uuid(changed_element)
+            xml_string = etree.tostring(cleaned_element, pretty_print=True).decode(encoding='UTF-8')
 
             random_name = str(int(random.random() * 1000000))
 
@@ -1131,7 +1227,7 @@ class Panoply:
             snippet['name'] = f'{tag}-{random_name}'
             snippet['xpath'] = set_xpath
             snippet['element'] = xml_string.strip()
-            snippet['full_xpath'] = full_relative_xpath
+            snippet['full_xpath'] = xpath
             snippets.append(snippet)
 
         return self.__order_snippets(snippets)
@@ -1147,7 +1243,8 @@ class Panoply:
         """
         ignored_xpaths = (
             '/config/mgt-config/users/entry[@name="admin"]',
-            '/config/shared/content-preview'
+            '/config/shared/content-preview',
+            '/config/readonly'
         )
 
         for ignored_xpath in ignored_xpaths:
@@ -1158,7 +1255,55 @@ class Panoply:
         return False
 
     @staticmethod
-    def generate_set_cli_from_configs(previous_config: str, latest_config: str) -> list:
+    def __is_ignored_set_cli(set_cli: str) -> bool:
+        """
+        Determines if the generated set cli can be used to recreate this configuration. Some generated
+        set clis are simply remnants of the XML structure and are not actually valid set cli commands
+
+        :param set_cli: set command to check
+        :return: bool True if this cli should be excluded
+        """
+
+        # ignore any set clis that start with this commands
+        ignored_set_prefix = (
+            'set mgt-config users admin phash',
+            'set shared content-preview',
+            'set read-only'
+        )
+
+        # ignore any set cli that match these commands
+        ignored_set_cli = (
+            'set shared application',
+            'set shared application-group',
+            'set shared service',
+            'set shared service-group'
+        )
+
+        # ignore any set cli that contain these strings
+        ignored_set_cli_parts = (
+            'deviceconfig setting management initcfg',
+            'deviceconfig setting management disable-predefined-reports'
+
+        )
+
+        for prefix in ignored_set_prefix:
+            if set_cli.startswith(prefix):
+                logger.debug(f'Skipping ignore set cli prefix: {set_cli}')
+                return True
+
+        for ignorned_set in ignored_set_cli:
+            if set_cli == ignorned_set:
+                logger.debug(f'Skipping ignore set cli: {set_cli}')
+                return True
+
+        for part in ignored_set_cli_parts:
+            if part in set_cli:
+                logger.debug(f'Skipping ignored set cli with part: {set_cli}')
+                return True
+
+        return False
+
+    def generate_set_cli_from_configs(self, previous_config: str, latest_config: str) -> list:
         """
         Takes two configuration files, converts them to set commands, then returns only the commands found
         in the 'latest_config' vs the 'previous_config'. This allows the user to quickly configure one firewall,
@@ -1180,13 +1325,15 @@ class Panoply:
 
         for cmd in l_set:
 
-            if cmd not in p_set and 'set readonly' not in cmd:
-                cmd_cleaned = cmd.replace('devices localhost.localdomain ', '') \
-                    .replace('\n', ' ') \
-                    .replace('vsys vsys1 ', '')
+            if cmd not in p_set:
+
+                if self.__is_ignored_set_cli(cmd):
+                    continue
+
+                cmd_cleaned = cmd.replace('\n', ' ')
                 diffs.append(cmd_cleaned)
 
-        return diffs
+        return self.__order_set_commands(diffs)
 
     def __check_element(self, el: etree.Element, xpath: str, pc: etree.Element, not_founds: list) -> list:
         """
@@ -1282,6 +1429,10 @@ class Panoply:
             # return our findings up the stack
             return not_founds
 
+        # check this element to determine if it's 'blank'
+        if len(el.findall('./')) == 0 and not el.attrib and (not el.text or not el.text.strip()):
+            return not_founds
+
         not_founds.append(xpath)
         return not_founds
 
@@ -1293,47 +1444,32 @@ class Panoply:
 
         return new_xpath, entry
 
-    def __order_snippets(self, snippets: list):
+    def __order_snippets(self, snippets: list) -> list:
+        """
+        Orders a set of snippets according to PAN-OS rules. This is an evolving list, which will almost certainly
+        be 100% correct. Please file issues if you find an ordering issue in auto-generated snippets or set commands
+
+        :param snippets: list of snippet dictionaries to order
+        :return:  list of a snippet dictionary in the order in which they need to be applied to a NGFW or Panorama
+        """
         # Attempt to order the snippets in a cohesive ordering. Will never be 100% perfect,
         # but at least make the attempt
 
-        xpaths = [
-            '/shared/certificate',
-            '/shared/ssl-tls-service-profile',
-            '/shared/tag',
-            '/shared/profiles',
-            '/shared/reports',
-            '/shared/',  # catch the rest of the shared items here
-            '/tag/entry',
-            '/deviceconfig/system',
-            '/network/profiles',
-            '/network/interface',
-            '/network/virtual-wire',
-            '/network/vlan',
-            '/network/ike',
-            '/network/tunnel',
-            '/import/network/interface'  # for use with panorama plugin import (SD-WAN)
-            '/network/virtual-router',
-            '/network/profiles/zone-protection-profile',
-            'routing-table/ip/static-route/entry/next-hop',  # try to keep next-hop before path-monitor for set cli
-            'routing-table/ip/static-route/entry/path-monitor',  # #70
-            '/dynamic-ip-and-port/interface-address',  # GH #70
-            '/dynamic-ip-and-port/interface',  # GH #70
-            '/dynamic-ip-and-port/ip',  # GH #70
-            '/zone/entry',
-            '/profiles/custom-url-category',  # should come before profiles/url-filtering
-            '/address/entry'  # should come before rules or address-group
-        ]
+        xpaths, post_xpaths = self.get_ordered_xpaths()
 
         ordered_snippets = list()
         for x in xpaths:
-            ordered_snippets.extend(self.__filter_snippets_by_xpath(snippets, x))
-
-        post_xpaths = ['/rulebase']
+            found_snippets = self.__filter_snippets_by_xpath(snippets, x)
+            for s in found_snippets:
+                if s not in ordered_snippets:
+                    ordered_snippets.append(s)
 
         post_snippets = list()
         for p in post_xpaths:
-            post_snippets.extend(self.__filter_snippets_by_xpath(snippets, p))
+            filtered_snippets = self.__filter_snippets_by_xpath(snippets, p)
+            for f in filtered_snippets:
+                if f not in post_snippets:
+                    post_snippets.append(f)
 
         for s in snippets:
             if s not in ordered_snippets and s not in post_snippets:
@@ -1345,20 +1481,61 @@ class Panoply:
 
         return ordered_snippets
 
+    def __order_set_commands(self, set_commands: list) -> list:
+        """
+        This will attempt to order a list of set commands using the same logic as ordering snippets.
+        This is done by converting the set command into a 'fake' xpath, creating a fake snippet dictionary
+        and sending those through the order_snippets method
+
+        :param set_commands: list of set commands to order
+        :return: a list of ordered set commands
+        """
+
+        ordered_set_commands = list()
+
+        fake_snippets = list()
+
+        for set_cmd in set_commands:
+            snippet = dict()
+            snippet['full_xpath'] = set_cmd.replace(' ', '/')
+            fake_snippets.append(snippet)
+
+        ordered_fake_snippets = self.__order_snippets(fake_snippets)
+
+        for fake_snippet in ordered_fake_snippets:
+            new_set_command = fake_snippet['full_xpath'].replace('/', ' ')
+            ordered_set_commands.append(new_set_command)
+
+        return ordered_set_commands
+
     @staticmethod
     def __filter_snippets_by_xpath(snippets: list, xpath: str) -> list:
         """
-        Check the each snippet in the list and return those that has a full_xpath matching the xpath param
+        Check the each snippet in the list and return those that has a full_xpath matching the xpath param. This is
+        done by finding the specific leaf node of the xpath by using only the xpath portions after any template, device,
+        or vsys entries. So an xpath pattern like 'network/interface' will match even within templates, across vsys,
+        etc
 
         :param snippets: list of snippets
         :param xpath: xpath to check
         :return: list of only those that contain the xpath in their full_xpath attribute
         """
         filtered_snippets = list()
+
+        split_pattern = re.compile(r'/devices/.*?/|vsys/.*?/')
+
         for s in snippets:
             full_xpath = s.get('full_xpath', '')
-            if xpath in full_xpath:
-                # snippets.remove(s)
+
+            split_xpath = split_pattern.split(full_xpath)
+            leaf_xpath_initial = split_xpath[-1]
+
+            # handle cases like ./mgt-config/password-complexity - remove the leading './'
+            leaf_xpath = re.sub(r'^\./', '', leaf_xpath_initial)
+            if leaf_xpath.startswith(xpath):
+                # note we do not remove found snippets from the source snippets list, which may result
+                # in duplicates. The calling code will need to ensure it does not append the results of this method
+                # which out checking for dups first
                 filtered_snippets.append(s)
 
         return filtered_snippets
@@ -1573,9 +1750,14 @@ class Panoply:
         if changed_element is None:
             return changed_element
 
+        # check if 'uuid' is an attribute on this element
+        if 'uuid' in changed_element.attrib:
+            changed_element.attrib.pop('uuid')
+
+        # this will find all child nodes with a 'uuid' attribute
         child_nodes = changed_element.findall('.//*[@uuid]')
 
-        if child_nodes is None:
+        if child_nodes is None or len(child_nodes) == 0:
             return changed_element
 
         for child in child_nodes:
@@ -1649,6 +1831,47 @@ class Panoply:
 
         return path
 
+    @staticmethod
+    def get_ordered_xpaths() -> tuple:
+        """
+        Returns a list of ordered xpaths for use in ordering snippets and set commands
+
+        Will be enhanced one day with version and model specific information if necessary
+
+        :return: tuple of two lists, xpaths and post_xpaths
+        """
+        xpaths = [
+            'shared/certificate',
+            'shared/ssl-tls-service-profile',
+            'shared/tag',
+            'shared/profiles',
+            'shared/reports',
+            'shared/',  # catch the rest of the shared items here
+            'tag/',
+            'deviceconfig/system',
+            'network/profiles',
+            'network/interface',
+            'network/virtual-wire',
+            'network/vlan',
+            'network/ike',
+            'network/tunnel',
+            'import/network/interface',  # for use with panorama plugin import (SD-WAN)
+            'network/virtual-router',
+            'network/profiles/zone-protection-profile',
+            'routing-table/ip/static-route/entry/next-hop',  # try to keep next-hop before path-monitor for set cli
+            'routing-table/ip/static-route/entry/path-monitor',  # #70
+            'dynamic-ip-and-port/interface-address',  # GH #70
+            'dynamic-ip-and-port/interface',  # GH #70
+            'dynamic-ip-and-port/ip',  # GH #70
+            'zone/',
+            'profiles/custom-url-category',  # should come before profiles/url-filtering
+            'address/'  # should come before rules or address-group
+        ]
+
+        post_xpaths = ['rulebase']
+
+        return xpaths, post_xpaths
+
 
 class EphemeralPanos(Panoply):
     """
