@@ -14,7 +14,7 @@
 
 # Authors: Adam Baumeister, Nathan Embery
 
-
+import copy
 import html
 import logging
 import os
@@ -26,8 +26,13 @@ from pathlib import Path
 from typing import Generator
 from typing import List
 
+import yaml
+from yaml.scanner import ScannerError
+
 from skilletlib.exceptions import SkilletLoaderException
+from skilletlib.exceptions import SkilletValidationException
 from skilletlib.snippet.base import Snippet
+from skilletlib.snippet.template import SimpleTemplateSnippet
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
@@ -113,7 +118,11 @@ class Skillet(ABC):
                 return html.unescape(sf.read())
 
         else:
-            raise SkilletLoaderException('Could not resolve template path!')
+            # Add the snippet name here as well to allow for more context
+            # fix for https://gitlab.com/panw-gse/as/panhandler/-/issues/19
+            logger.error(f'Snippet: {self.name} has file attribute that does not exist')
+            logger.error(f'Snippet file path is: {template_path}')
+            raise SkilletLoaderException(f'Snippet: {self.name} - Could not resolve template path!')
 
     def update_context(self, d: dict) -> dict:
         """
@@ -342,6 +351,18 @@ class Skillet(ABC):
 
         :return: dictionary of results from the Skillet execute or execute_async method
         """
+
+        results = self._get_snippet_results()
+
+        results['outputs'] = self.captured_outputs
+        return self._parse_output_template(results)
+
+    def _get_snippet_results(self) -> dict:
+        """
+        Iterate the snippets in the skillet and collect their combined results into the 'results' dict
+
+        :return: dict with 'snippets' key
+        """
         results = dict()
         results['snippets'] = dict()
 
@@ -351,8 +372,32 @@ class Skillet(ABC):
             if snippet_name in self.snippet_outputs:
                 results['snippets'][snippet_name] = self.snippet_outputs[snippet_name]
 
-        results['outputs'] = self.captured_outputs
-        # results.update(self.captured_outputs)
+        return results
+
+    def _parse_output_template(self, results: dict) -> dict:
+        """
+        Check for the presence of the output_template label. If found, load and parse the template
+        using the results. The parsed output will be placed in the 'output_template' key in the results
+
+        :param results: results of the skillet execution
+        :return: results plus the output_template value if found
+        """
+        try:
+            if 'output_template' in self.labels:
+                output_template = self.load_template(self.labels['output_template'])
+                template_snippet = SimpleTemplateSnippet(output_template)
+                # create context dict for template parsing
+                # add all outputs as 'top-level' attributes
+                context = results.get('outputs', {})
+                # add other keys as top-level attributes as well...
+                for k, v in results.items():
+                    if k != 'outputs':
+                        context[k] = v
+                results['output_template'] = template_snippet.template(context)
+
+        except SkilletLoaderException as sle:
+            print(sle)
+
         return results
 
     def __validate_snippet_metadata(self) -> None:
@@ -378,16 +423,19 @@ class Skillet(ABC):
 
     def get_declared_variables(self) -> List[str]:
         """
-        Return a list of all variables defined in all the snippets
+        Return a list of all variables defined in all the snippets that are not defined as an output
 
         :return: list of variable names
         """
 
-        declared_variables = list()
-        for s in self.snippets:
-            declared_variables.extend(s.get_snippet_variables())
+        # get list of output_vars from all snippets using double list comprehension
+        output_vars = [o for s in self.get_snippets() for o in s.get_output_variables()]
 
-        return declared_variables
+        # get list of all variables defined in all snippets that are NOT in the output_vars
+        dv = [x for s in self.get_snippets() for x in s.get_snippet_variables() if x not in output_vars]
+
+        # convert to set and back to list to remove dups
+        return list(set(dv))
 
     @staticmethod
     def __normalize_skillet_dict(skillet: dict) -> dict:
@@ -424,3 +472,33 @@ class Skillet(ABC):
             skillet['labels']['collection'].append('Kitchen Sink')
 
         return skillet
+
+    def dump_yaml(self) -> str:
+        """
+        Convert this Skillet into a YAML formatted string
+
+        :return: YAML formatted string
+        """
+        try:
+
+            # remove non-essential non-portable items from the dict before dumping
+            safe_skillet_dict = copy.deepcopy(self.skillet_dict)
+
+            safe_skillet_dict.pop('snippet_path', None)
+            safe_skillet_dict.pop('app_data', None)
+
+            # source https://stackoverflow.com/a/45004775
+            yaml.SafeDumper.org_represent_str = yaml.SafeDumper.represent_str
+
+            def repr_str(dumper, data):
+                if '\n' in data:
+                    return dumper.represent_scalar(u'tag:yaml.org,2002:str', data, style='|')
+
+                return dumper.org_represent_str(data)
+
+            yaml.add_representer(str, repr_str, Dumper=yaml.SafeDumper)
+
+            return yaml.safe_dump(safe_skillet_dict, indent=4)
+
+        except (ScannerError, ValueError) as err:
+            raise SkilletValidationException(f'Could not dump Skillet as YAML: {err}')
