@@ -180,13 +180,7 @@ class PanosSkillet(Skillet):
 
             if 'cmd' not in snippet_def or snippet_def['cmd'] == 'set':
                 if 'element' not in snippet_def or snippet_def['element'] == '':
-                    try:
-                        snippet_def['element'] = self.load_template(snippet_def['file'])
-                    except SkilletLoaderException as sle:
-                        # Add the snippet name here as well to allow for more context
-                        logger.error(f'Snippet: {snippet_def["name"]} has file attribute that does not exist')
-                        logger.error(f'Snippet file path is: {snippet_def["file"]}')
-                        raise SkilletLoaderException(f'Snippet: {snippet_def["name"]} - {sle}')
+                    snippet_def['element'] = self.load_template(snippet_def['file'])
 
             snippet = PanosSnippet(snippet_def, self.panoply)
             snippet_list.append(snippet)
@@ -279,7 +273,9 @@ class PanosSkillet(Skillet):
         :return: dict containing default outputs plus the overall result and changed flag
         """
 
-        results = super().get_results()
+        results = super()._get_snippet_results()
+        results['outputs'] = self.captured_outputs
+
         skillet_result = 'success'
 
         changed = False
@@ -298,4 +294,4 @@ class PanosSkillet(Skillet):
         results['result'] = skillet_result
         results['changed'] = changed
 
-        return results
+        return self._parse_output_template(results)
