@@ -442,17 +442,30 @@ class Snippet(ABC):
         parsed_template_str = self._env.parse(template_str)
         return meta.find_undeclared_variables(parsed_template_str)
 
+    def get_output_variables(self) -> list:
+        """
+        Returns a list of all output variables. This is used to determine if a snippet variable should be considered
+        undeclared.
+
+        :return: list of str representing output variable names
+        """
+
+        return [x['name'] for x in self.metadata.get('outputs', dict()) if 'name' in x]
+
     def get_snippet_variables(self) -> list:
         """
-        Returns a list of variables defined in this snippet
+        Returns a list of variables defined in this snippet that are NOT defined as outputs
 
-        :return: list of str representing variable found in the jinja templates
+        :return: list of str representing variables found in the jinja templates
         """
 
         variables = list()
         for i in self.template_metadata:
             if i in self.metadata:
-                variables.extend(self.get_variables_from_template(self.metadata[i]))
+                found_vars = self.get_variables_from_template(self.metadata[i])
+                for f in found_vars:
+                    if f not in variables:
+                        variables.append(f)
 
         return variables
 
@@ -597,6 +610,11 @@ class Snippet(ABC):
                 # no matches should return an empty list
                 outputs[output_name] = list()
 
+        elif 'capture_expression' in output_definition:
+            value = str(self.render(output_definition['capture_expression'], self.context))
+
+            outputs[output_name] = value
+
         else:
             output_name = output_definition.get('name', self.name)
             outputs[output_name] = results
@@ -742,6 +760,11 @@ class Snippet(ABC):
 
                 captured_output[var_name] = capture_list
 
+            elif 'capture_expression' in output_definition:
+                value = str(self.render(output_definition['capture_expression'], self.context))
+
+                captured_output[var_name] = value
+
             # filter selected items here
             captured_output[var_name] = self.__filter_outputs(output, captured_output[var_name], local_context)
 
@@ -804,6 +827,13 @@ class Snippet(ABC):
         output = self.__render_output_metadata(output_definition, local_context)
 
         try:
+            if 'capture_expression' in output_definition:
+                var_name = output_definition['name']
+                value = str(self.render(output_definition['capture_expression'], self.context))
+
+                captured_output[var_name] = value
+                return captured_output
+
             for i in ('capture_pattern', 'capture_value', 'capture_object'):
                 if i in output:
                     capture_pattern = output[i]
